/**
 * Creates a new JSX element with the specified properties and children. Supports everything in the JSX syntax.
 *
 * Note: `className` will transpile to HTML `class`
 * @example
 * const element1 = <a></a>;
 * const element2 = <div className="element wrapper"></div>;
 * @param name The name of the element. (Like `div`, `a`, etc.)
 * @param props The properties/attributes assigned to the element.
 * @param children Children of the element.
 * @returns The created element.
 */
export declare function createElement(name: string, props: Props, ...children: Node[]): Element;
/**
 * Creates a new JSX fragment.
 *
 * A fragment is just an element without a name, but when inserted somewhere in the element tree, it disappears.
 * @example
 * const fragment =
 *     <>
 *         <div>Element 1</div>
 *         <div>Element 2</div>
 *     </>
 * const app =
 *     <div>
 *         some text
 *         {fragment}
 *         some more text
 *     </div>
 * // equivalent to
 * const app =
 *     <div>
 *         some text
 *         <div>Element 1</div>
 *         <div>Element 2</div>
 *         some more text
 *     </div>
 * @param children The elements in the fragment.
 * @returns The newly created fragment.
 */
export declare function createElement(name: Fragment, props: Props, ...children: Node[]): Element;
/**
 * Calls the generator function with the corresponding data and returns the generated element.
 * @example
 * const Block = ({ props, children }) => (
 *     <div class="block" {...props}>
 *         <div class="blockHeader">{props.title}</div>
 *         <div>{children}</div>
 *     </div> // or className
 * );
 * const myBlock = <Block title="A block">
 *     <a href="">Link</a>
 * </Block>;
 * @param generator Element generator function.
 * @param props The properties/attributes assigned to the element.
 * @param children Children passed to the element generator.
 * @returns The created element.
 */
export declare function createElement<T extends Props>(generator: ElementGenerator<T>, props: T, ...children: Node[]): Element;
/**
 * Readonly data container for JSX elements.
 */
export declare class Element {
    name: string | Fragment | typeof HTML | typeof CASE | typeof DEFAULT | typeof HEAD;
    props: Props;
    children: Node[];
    /**
     * Creates a new element data container.
     * @param name The name of the element. (Like `div`, `a`, etc.)
     * @param props The properties/attributes assigned to the element.
     * @param children Children of the element.
     */
    constructor(name: string | Fragment | typeof HTML | typeof CASE | typeof DEFAULT | typeof HEAD, props: Props, children: Node[]);
}
/**
 * Renders the JSX elements to a string. This function provides the core functionallity.
 * @param top The first (top) element that should be rendered.
 * @param options Options to use for rendering.
 * @param indentation The starting indentation level. Only used when options.indent is set to `true`.
 */
export declare function render(top: Element, options?: RenderOptions, indentation?: number): string;
/**
 * Options for the rendering.
 */
export interface RenderOptions {
    /**
     * This option specifies if the resulting code should be formated and indented. If it is set to `false`, the whole output will be in one line.
     */
    indent?: boolean;
    /**
     * Sets the amount of whitespaces used for indenting. Only used when `options.indent` is true.
     */
    indentString?: string;
    /**
     * If you don't want to use whitespaces, if you want to use tabs for example, you can set this to your indentation string. If both `options.indentString` and `options.indentSize` are set, this one (`options.indentString`) will be prefered.
     */
    indentSize?: number;
    /**
     * When you don't have children in an element, it will be outputed like `<div />`. If this is set to false, it will normally output both tags. If you even specify a list of strings, the self-closing will only be rendered for tags in that list. This is very useful when rendering web pages with `img` or `input` elements inside.
     */
    useSelfCloseTags?: boolean | string[];
}
export declare function create(options?: RenderOptions): ExpressRenderer;
/**
 * Symbol indicating that the element is a fragment.
 */
export declare const Fragment: unique symbol;
/**
 * Element for an HTML page (including DOCTYPE, head and body).
 */
export declare const HtmlPage: ElementGenerator<{
    [key: string]: any;
    status?: number;
}>;
declare const HTML: unique symbol;
/**
 * Helper component for conditional elements.
 */
export declare const If: ElementGenerator<{
    cond: boolean;
}>;
/**
 * Helper component for multiple case szenarios.
 */
export declare const Switch: ElementGenerator<{
    expr: any;
}>;
export declare const Case: ElementGenerator<{
    c: any;
}>;
export declare const Default: ElementGenerator<{}>;
declare const CASE: unique symbol;
declare const DEFAULT: unique symbol;
export declare const Head: ElementGenerator;
declare const HEAD: unique symbol;
/**
 * Element attributes/properties type.
 */
export declare type Props = {
    [key: string]: any;
};
/**
 * Type for an element fragment.
 */
export declare type Fragment = typeof Fragment;
/**
 * Node (JSX element like text, elements, and arrays) type.
 */
export declare type Node = Element | string | Node[];
/**
 * Function for generating elements.
 */
export declare type ElementGenerator<T extends Props = {}> = (props: T, children: Node[]) => Element;
/**
 * Return type for the `jsxt.create` function.
 */
export interface ExpressRenderer {
    (top: Element): string;
    createHandler(component: ElementGenerator<{
        req: ExpressRequest;
        res: ExpressResponse;
    }>, status?: number): (req: ExpressRequest, res: ExpressResponse) => void;
}
import type { Request as ExpressRequest, Response as ExpressResponse } from "express";
declare global {
    export namespace JSX {
        interface IntrinsicElements {
            [key: string]: any;
        }
    }
}
declare const _default: {
    createElement: typeof createElement;
    Element: typeof Element;
    render: typeof render;
    Fragment: symbol;
    HtmlPage: ElementGenerator<{
        [key: string]: any;
        status?: number;
    }>;
    create: typeof create;
};
export default _default;
